# Generated by Django 3.2.13 on 2022-07-02 15:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_blocklist", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="blockedip",
            name="tally",
            field=models.IntegerField(
                default=0, help_text="Number of times this IP has been blocked since first_seen"
            ),
        ),
    ]
